--[[

BhaldieMoveIT v2.3.3
	
	BhaldieMoveIT: Prefect Flexbar companion. This mod is prefect for flexbar it breaks up many components 
			on the UI interface and makes them moveable.  This is every useful with Flexbar because
			you can put your buttons wherever you want, and move or disable the other UI components
			to match your Flexbar setup, not the other way around.

	- Patch notes:
		- UI Menu Update
			
	- BhaldieMoveIT v2.1 Features:
		- UI Window or Slash Command still available.
		- Disable/Enable and Moveable Bag Bar.
		- Disable/Enable and Moveable Charter Mini Bar.
		- Disable/Enable and Moveable Pet Bar.
		- Disable/Enable Default Action Menu.
		- Disable/Enable and Moveable Bag Status Bar.
		- Moveable Shape Shift Bar.
		- Moveable Cast Bar.
		- Moveable Bags (Option to disable).
]]

function BhaldieMoveIT_Onload()	
	this:RegisterEvent("VARIABLES_LOADED");

	WoW_PetActionBarFrame_OnUpdate = PetActionBarFrame_OnUpdate;	
	PetActionBarFrame_OnUpdate = BM_PetActionBarFrame_OnUpdate;

	WoW_CastingBarFrame_OnUpdate = CastingBarFrame_OnUpdate;	
	CastingBarFrame_OnUpdate = BM_CastingBarFrame_OnUpdate;
	
	WoW_ShapeshiftBar_Update = ShapeshiftBar_Update;	
	ShapeshiftBar_Update = BM_ShapeshiftBar_Update;
	
	WoW_UIParent_ManageFramePositions = UIParent_ManageFramePositions;	
	UIParent_ManageFramePositions = BM_UIParent_ManageFramePositions;
	
end 

-- Set the capsule backdrop if it shows
function BhaldieMoveITArtOnShow ()
	this:SetBackdropBorderColor(0, 0, 0);
	this:SetBackdropColor(0, 0, 0);
end

-- Sets the position of frames when frame is locked through slash commands.
function BhaldieSetPosition(BhaldieFrame)
	local x = BhaldieFrame:GetRight();
	local y = BhaldieFrame:GetBottom();
	BhaldieFrame:ClearAllPoints();
	BhaldieFrame:SetPoint("BOTTOMRIGHT", "UIParent", "BOTTOMLEFT", x, y);
end

function BhaldieMoveIT_OnEvent(event)
	if (event == "VARIABLES_LOADED") then		
		BhaldieMoveIT_initialize();
		return;
	end
end

function BhaldieMoveIT_initialize()
	-- add the realm to the "player's name" for the config settings
	BM_PLAYERNAME_REALM = GetCVar("realmName") .. "|" .. UnitName("player");
	
	-- Make sure BM_CONFIG is ready
	if (not BM_CONFIG) then
		BM_CONFIG = { };
	end
	-- Check if there's not an entry for this player
	if (not BM_CONFIG[BM_PLAYERNAME_REALM]) then
		BM_CONFIG[BM_PLAYERNAME_REALM] = {};
	end
	-- set the defaults
	-- Action Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].action_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].action_enable = DEFAULT_BM_ACTION_ENABLE;
	end
	-- Bag Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_lock = DEFAULT_BM_BAG_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_enable = DEFAULT_BM_BAG_ENABLE;
	end
	-- Mini Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].minibar_lock = DEFAULT_BM_MINI_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].minibar_enable = DEFAULT_BM_MINI_ENABLE;
	end
	-- Cast Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].castbar_lock = DEFAULT_BM_CAST_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].castbar_enable = true;
	end
	-- Pet Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].petbar_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].petbar_lock = DEFAULT_BM_PET_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].petbar_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].petbar_enable = DEFAULT_BM_PET_ENABLE;
	end
	-- Bonus Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_lock = DEFAULT_BM_BONUS_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_enable = true;
	end
	-- Bag Status Meter
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_lock == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_lock = DEFAULT_BM_BSTATUS_LOCK;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_enable = Default_BhaldieBagStatus_Enable;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_enable = DEFAULT_BM_BSTATUS_ENABLE;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].mail_frame == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].mail_frame = DEFAULT_BM_MAIL_FRAME;
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].movebags_enable == nil) then
		BM_CONFIG[BM_PLAYERNAME_REALM].movebags_enable = true;
	end

	-- show or hide the right options
	if (BM_CONFIG[BM_PLAYERNAME_REALM].action_enable == false) then
		MainMenuBar:ClearAllPoints();
		MainMenuBar:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -500);
	else
		MainMenuBar:ClearAllPoints();
		MainMenuBar:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, 0);
	end
	
	-- Bag bar lock unlock show and hide.
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_lock == true) then
		BhaldieBagBar:Hide();
		BhaldieBagBarArt:Hide();
	else
		BhaldieBagBar:Show();
		BhaldieBagBarArt:Show();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_enable == false) then
		MainMenuBarBackpackButton:Hide();
		CharacterBag0Slot:Hide();
		CharacterBag1Slot:Hide();
		CharacterBag2Slot:Hide();
		CharacterBag3Slot:Hide();
		BhaldieBagBar:Hide();
		BhaldieBagBarArt:Hide();
		KeyRingButton:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].bagbar_lock == false) then
			BhaldieBagBar:Show();
			BhaldieBagBarArt:Show();
		else
			BhaldieBagBar:Hide();
			BhaldieBagBarArt:Hide();
		end
		MainMenuBarBackpackButton:Show();
		CharacterBag0Slot:Show();
		CharacterBag1Slot:Show();
		CharacterBag2Slot:Show();
		CharacterBag3Slot:Show();
		if (SHOW_KEYRING == 1) then
			MainMenuBarTexture3:SetTexture("Interface\\MainMenuBar\\UI-MainMenuBar-KeyRing");
			MainMenuBarTexture3:SetTexCoord(0, 1, 0.1640625, 0.5);
			MainMenuBarTexture2:SetTexture("Interface\\MainMenuBar\\UI-MainMenuBar-KeyRing");
			MainMenuBarTexture2:SetTexCoord(0, 1, 0.6640625, 1);
			MainMenuBarPerformanceBarFrame:SetPoint("BOTTOMRIGHT", MainMenuBar, "BOTTOMRIGHT", -235, -10);
			KeyRingButton:Show();
		end
	end
	
	-- Mini bar lock unlock show and hide.
	if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_enable == true) then
		if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_lock == false) then
			BhaldieMiniBar:Hide();
			BhaldieMiniBarArt:Hide();
		else
			BhaldieMiniBar:Show();
			BhaldieMiniBarArt:Show();
		end
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_enable == false) then
		BM_MINI_ENABLE = false;
		CharacterMicroButton:Hide();
		SpellbookMicroButton:Hide();
		QuestLogMicroButton:Hide();
		SocialsMicroButton:Hide();
		WorldMapMicroButton:Hide();
		MainMenuMicroButton:Hide();
		HelpMicroButton:Hide();
		BhaldieMiniBar:Hide();
		BhaldieMiniBarArt:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].minibar_lock == false) then
			BhaldieMiniBar:Show();
			BhaldieMiniBarArt:Show();
		else
			BhaldieMiniBar:Hide();
			BhaldieMiniBarArt:Hide();
		end
		BM_MINI_ENABLE = true;
		CharacterMicroButton:Show();
		TalentMicroButton:Show();
		SpellbookMicroButton:Show();
		QuestLogMicroButton:Show();
		SocialsMicroButton:Show();
		WorldMapMicroButton:Show();
		MainMenuMicroButton:Show();
		HelpMicroButton:Show();
	end
	
	-- Cast bar lock unlock.
	if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_lock == true) then
		BhaldieCastBar:Hide();
		BhaldieCastBarArt:Hide();
	else
		BhaldieCastBar:Show();
		BhaldieCastBarArt:Show();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_enable == false) then
		BhaldieCastBar:Hide();
		BhaldieCastBarArt:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_lock == false) then
			BhaldieCastBar:Show();
			BhaldieCastBarArt:Show();
		else
			BhaldieCastBar:Hide();
			BhaldieCastBarArt:Hide();
		end
	end
	
	-- Pet bar lock unlock hide show.
	if (BM_CONFIG[BM_PLAYERNAME_REALM].petbar_lock == true) then
		BhaldiePetBar:Hide();
		BhaldiePetBarArt:Hide();
	else
		BhaldiePetBar:Show();
		BhaldiePetBarArt:Show();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].petbar_enable == false) then
		PetActionButton1:Hide();
		PetActionButton2:Hide();
		PetActionButton3:Hide();
		PetActionButton4:Hide();
		PetActionButton5:Hide();
		PetActionButton6:Hide();
		PetActionButton7:Hide();
		PetActionButton8:Hide();
		PetActionButton8:Hide();
		PetActionButton10:Hide();
		BhaldiePetBar:Hide();
		BhaldiePetBarArt:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].petbar_lock == false) then
			BhaldiePetBar:Show();
			BhaldiePetBarArt:Show();
		else
			BhaldiePetBar:Hide();
			BhaldiePetBarArt:Hide();
		end
		PetActionButton1:Show();
		PetActionButton2:Show();
		PetActionButton3:Show();
		PetActionButton4:Show();
		PetActionButton5:Show();
		PetActionButton6:Show();
		PetActionButton7:Show();
		PetActionButton8:Show();
		PetActionButton8:Show();
		PetActionButton10:Show();
	end
	
	-- Bonus Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_lock == true) then
		BhaldieShapeShift:Hide();
		BhaldieShapeShiftArt:Hide();
	else
		BhaldieShapeShift:Show();
		BhaldieShapeShiftArt:Show();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_enable == false) then
		BhaldieShapeShift:Hide();
		BhaldieShapeShiftArt:Hide();
		ShapeshiftBarFrame:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_lock == false) then
			BhaldieShapeShift:Show();
			BhaldieShapeShiftArt:Show();
		else
			BhaldieShapeShift:Hide();
			BhaldieShapeShiftArt:Hide();
		end
		ShapeshiftBarFrame:Show();
	end
	
	-- Bag Status Bar
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_lock == true) then
		BhaldieBagStatusBar:Hide();
	else
		BhaldieBagStatusBar:Show();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_enable == false) then
		BhaldieBagStatusBar:Hide();
		BagStatusBarFrame:Hide();
	else
		if (BM_CONFIG[BM_PLAYERNAME_REALM].bagstatus_lock == false) then
			BhaldieBagStatusBar:Show();
		else
			BhaldieBagStatusBar:Hide();
		end
		BagStatusBarFrame:Show();
	end
	
	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage(BHALDIEMOVEIT_LOADED);
		UIErrorsFrame:AddMessage(BHALDIEMOVEIT_LOADED, 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME);
	end

	if (BM_CONFIG[BM_PLAYERNAME_REALM].mail_frame == true) then
		--Moves the mailframe to another location on the mini map.
		MiniMapMailFrame:ClearAllPoints();
		MiniMapMailFrame:SetPoint("BOTTOM", "Minimap", "LEFT", -8, -24);
	else
		--Moves the mailframe to default location on the mini map.
		MiniMapMailFrame:ClearAllPoints();
		MiniMapMailFrame:SetPoint("TOPRIGHT", "Minimap", "TOPRIGHT", 21, -38);
	end

	-- Creates BhaldieBagBar
	MainMenuBarBackpackButton:ClearAllPoints();
	CharacterBag0Slot:ClearAllPoints();
	MainMenuBarBackpackButton:SetPoint("BOTTOMRIGHT", "BhaldieBagBar", "BOTTOMRIGHT", -15, 6);
	CharacterBag0Slot:SetPoint("BOTTOMRIGHT", "MainMenuBarBackpackButton", "BOTTOMLEFT", -4, 0);
	
	-- Creates BhaldieMiniBar
	CharacterMicroButton:ClearAllPoints();
	CharacterMicroButton:SetPoint("BOTTOMLEFT", "BhaldieMiniBar", "BOTTOMLEFT", 6, 6);
	
	--PetActionButton1:Show();
	PetActionButton1:ClearAllPoints();
	PetActionButton1:SetPoint("BOTTOMLEFT", "BhaldiePetBar", "BOTTOMLEFT", 6, 8);
	-- Hides the PetActionBarFrame.
	PetActionBarFrame:ClearAllPoints();
	PetActionBarFrame:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMLEFT", 0, -50);
	
	-- Creates BhaldieBagStatusBar
	BagStatusBarFrame:ClearAllPoints();
	BagStatusBarFrame:SetPoint("BOTTOMLEFT", "BhaldieBagStatusBar", "BOTTOMLEFT", 1, 3);
	
	-- Creates BhaldieShapeShiftBar
	ShapeshiftBarLeft:Hide();
	ShapeshiftBarMiddle:Hide();
	ShapeshiftBarRight:Hide();
	ShapeshiftBarFrame:ClearAllPoints();
	ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "BhaldieShapeShift", "BOTTOMLEFT", 3, 8);

	-- Creates BhaldieCastBar
	CastingBarFrame:ClearAllPoints();
	CastingBarFrame:SetPoint("BOTTOM", "BhaldieCastBar", "BOTTOM", 0, 0);
	--[[
	CastingBarFlash:ClearAllPoints();
	CastingBarFlash:SetPoint("CENTER", "BhaldieCastBar", "CENTER", 0, -5);
	CastingBarFrameStatusBar:ClearAllPoints();
	CastingBarFrameStatusBar:SetPoint("CENTER", "BhaldieCastBar", "CENTER", 0, -5);
	CastingBarText:ClearAllPoints();
	CastingBarText:SetPoint("CENTER", "BhaldieCastBar", "CENTER", 0, -5);
	]]

	-- Clears all the points of the container frames.
	ContainerFrame1:ClearAllPoints();
	ContainerFrame2:ClearAllPoints();
	ContainerFrame3:ClearAllPoints();
	ContainerFrame4:ClearAllPoints();
	ContainerFrame5:ClearAllPoints();
	ContainerFrame6:ClearAllPoints();
	ContainerFrame7:ClearAllPoints();
	ContainerFrame8:ClearAllPoints();
	ContainerFrame9:ClearAllPoints();
	ContainerFrame10:ClearAllPoints();
	ContainerFrame11:ClearAllPoints();
	ContainerFrame12:ClearAllPoints();
	
	--Removes PetActionBarFrame
	PetActionBarFrame:Hide();
	PetActionBarFrame:ClearAllPoints();
	PetActionBarFrame:SetPoint("BOTTOM", "UIParent", "BOTTOM", 0, -500);
	BM_UIParent_ManageFramePositions();
end
-- Loads all the moveable bars.
function BhaldieMoveIT_OnUpdate(arg1)
	BM_REFRESH = BM_REFRESH + arg1;
	if (BM_REFRESH > BM_DELAY) then
		-- Hides talentbutton on minibar.
		if (BM_MINI_ENABLE == false) then
			TalentMicroButton:Hide();
		end
		BM_REFRESH = 0.0;
		BM_BStatus_Meter_OnUpdate();
	end
	if (BM_CONFIG[BM_PLAYERNAME_REALM].movebags_enable) then
		BM_Moveable_Bags_OnUpdate();
	end
	return;
end

function BM_Moveable_Bags_OnUpdate()
	for i=0, 10, 1 do
		Container_Frame = getglobal("ContainerFrame" .. i+1);
		Container_Frame_ID[i] = Container_Frame:GetID();
		if (Container_Frame:IsVisible()) then
			Container_Frame:ClearAllPoints();
			Container_Frame:SetPoint("TOPLEFT", "BhaldieMoveBag" .. Container_Frame_ID[i], "TOPLEFT", 0, 0);
			--Container_Frame:SetFrameLevel(i+1);
			BM_CONTAINER_FRAME = getglobal("BhaldieMoveBag" .. Container_Frame_ID[i]);
			BM_CONTAINER_FRAME:Show();
		elseif (not Container_Frame:IsVisible() and Container_Frame_ID[i] < 12) then	
			Container_Frame_ID[i] = Container_Frame:GetID();
			BM_CONTAINER_FRAME = getglobal("BhaldieMoveBag" .. Container_Frame_ID[i]);
			BM_CONTAINER_FRAME:Hide();
			Container_Frame:SetID(100);
		end
	end
	return;
end

function BM_BStatus_Meter_OnUpdate()
	BM_TOTAL_BAGS = 0;
	BM_COUNT_USED_SLOTS = 0;
	BM_STATUS_BAR_COLOR = 0;
	BM_TOTAL_SLOTS_BAG = 0;
	BM_SLOTS_BAG = 0;
	BM_BAG_NAME = "";
	BM_BAG_SLOT_COUNT = 1;
	BM_IS_AMMO = false;
	for BM_SLOTS_BAG = 0, 4, 1 do
		BM_IS_AMMO = false;
		BM_TOTAL_SLOTS_BAG = 0;
		BM_BAG_NAME = GetBagName(BM_SLOTS_BAG);
		if (BM_BAG_NAME ~= nil) then
			for i = 0, table.getn(BM_IS_AMMO_BAG), 1 do
				if (string.find(BM_IS_AMMO_BAG[i], BM_BAG_NAME)) then
					BM_IS_AMMO = true;
				end
			end
			if (BM_IS_AMMO == false) then
				BM_TOTAL_BAGS = BM_TOTAL_BAGS + GetContainerNumSlots(BM_SLOTS_BAG);
				BM_TOTAL_SLOTS_BAG = GetContainerNumSlots(BM_SLOTS_BAG);
			end
			for BM_BAG_SLOT_COUNT = 1, BM_TOTAL_SLOTS_BAG, 1 do
				if (GetContainerItemInfo(BM_SLOTS_BAG, BM_BAG_SLOT_COUNT) and BM_IS_AMMO == false) then
					BM_COUNT_USED_SLOTS = BM_COUNT_USED_SLOTS + 1;
				end
			end
		end
	end
	--Sets the Bar size and color
	BagStatusBar:SetMinMaxValues(min(0, BM_COUNT_USED_SLOTS), max(0,BM_TOTAL_BAGS));
	BagStatusBar:SetValue(BM_COUNT_USED_SLOTS);
	BM_STATUS_BAR_COLOR = (BM_COUNT_USED_SLOTS/BM_TOTAL_BAGS)*100;
	if (BM_STATUS_BAR_COLOR >= 0 and BM_STATUS_BAR_COLOR <= 50) then
		BagStatusBar:SetStatusBarColor(0.0, 0.70, 0.0);
		BhaldieBagStatusBarText2:SetText(format(BAG_STATUS_METER_WHITE, BM_COUNT_USED_SLOTS, BM_TOTAL_BAGS));
	elseif (BM_STATUS_BAR_COLOR >= 51 and BM_STATUS_BAR_COLOR <= 75) then
		BagStatusBar:SetStatusBarColor(0.90, 0.90, 0.0);
		BhaldieBagStatusBarText2:SetText(format(BAG_STATUS_METER_WHITE, BM_COUNT_USED_SLOTS, BM_TOTAL_BAGS));
	elseif (BM_STATUS_BAR_COLOR >= 76 and BM_STATUS_BAR_COLOR <= 100) then
		BagStatusBar:SetStatusBarColor(0.95, 0.0, 0.0);
		BhaldieBagStatusBarText2:SetText(format(BAG_STATUS_METER_WHITE, BM_COUNT_USED_SLOTS, BM_TOTAL_BAGS));
	else
		BagStatusBar:SetStatusBarColor(0.0, 0.0, 0.90);
		BhaldieBagStatusBarText2:SetText(format(BAG_STATUS_METER_WHITE, BM_COUNT_USED_SLOTS, BM_TOTAL_BAGS));
	end

	BhaldieBagStatusBarText1:SetTextColor(1.0, 1.0, 1.0);
	return;
end

function BM_PetActionBarFrame_OnUpdate(elapsed)
	local yPos;
	if ( this.slideTimer and (this.slideTimer < this.timeToSlide) ) then
		SlidingActionBarTexture0:Hide();
		SlidingActionBarTexture1:Hide();
		this.completed = nil;
		if ( this.mode == "show" ) then
			yPos = (this.slideTimer/this.timeToSlide) * this.yTarget;
			this:SetPoint("TOPLEFT", this:GetParent(), "BOTTOMLEFT", PETACTIONBAR_XPOS, yPos);
			this.state = "showing";
			this:Show();
		elseif ( this.mode == "hide" ) then
			yPos = (1 - (this.slideTimer/this.timeToSlide)) * this.yTarget;
			this:SetPoint("TOPLEFT", this:GetParent(), "BOTTOMLEFT", PETACTIONBAR_XPOS, yPos);
			this.state = "hiding";
		end
		this.slideTimer = this.slideTimer + elapsed;
	else
		SlidingActionBarTexture0:Hide();
		SlidingActionBarTexture1:Hide();
		this.completed = 1;
		if ( this.mode == "show" ) then
			this:SetPoint("TOPLEFT", this:GetParent(), "BOTTOMLEFT", PETACTIONBAR_XPOS, this.yTarget);
			this.state = "top";
			--Move the chat frame and edit box up a bit
		elseif ( this.mode == "hide" ) then
			this:SetPoint("TOPLEFT", this:GetParent(), "BOTTOMLEFT", PETACTIONBAR_XPOS, 0);
			this.state = "bottom";
			this:Hide();
			--Move the chat frame and edit box back down to original position
		end
		this.mode = "none";
	end
end

function BM_CastingBarFrame_OnUpdate()
	if (BM_CONFIG[BM_PLAYERNAME_REALM].castbar_enable) then
		if ( this.casting ) then
			local status = GetTime();
			if ( status > this.maxValue ) then
				status = this.maxValue
			end
			CastingBarFrameStatusBar:SetValue(status);
			CastingBarFlash:Hide();
			local sparkPosition = ((status - this.startTime) / (this.maxValue - this.startTime)) * 195;
			if ( sparkPosition < 0 ) then
				sparkPosition = 0;
			end
			
			CastingBarFrame:ClearAllPoints();
			CastingBarFrame:SetPoint("BOTTOM", "BhaldieCastBar", "BOTTOM", 0, 8);
			CastingBarSpark:SetPoint("CENTER", CastingBarFrame, "LEFT", sparkPosition, 2);
		elseif ( this.channeling ) then
			local time = GetTime();
			if ( time > this.endTime ) then
				time = this.endTime
			end
			if ( time == this.endTime ) then
				this.channeling = nil;
				this.fadeOut = 1;
				return;
			end
			local barValue = this.startTime + (this.endTime - time);
			CastingBarFrameStatusBar:SetValue( barValue );
			CastingBarFlash:Hide();
			local sparkPosition = ((barValue - this.startTime) / (this.endTime - this.startTime)) * 195;
			CastingBarFrame:ClearAllPoints();
			CastingBarFrame:SetPoint("BOTTOM", "BhaldieCastBar", "BOTTOM", 0, 8);
			CastingBarSpark:SetPoint("CENTER", CastingBarFrame, "LEFT", sparkPosition, 2);
		elseif ( GetTime() < this.holdTime ) then
			return;
		elseif ( this.flash ) then
			local alpha = CastingBarFlash:GetAlpha() + CASTING_BAR_FLASH_STEP;
			if ( alpha < 1 ) then
				CastingBarFlash:SetAlpha(alpha);
			else
				CastingBarFlash:SetAlpha(1.0);
				this.flash = nil;
			end
		elseif ( this.fadeOut ) then
			local alpha = this:GetAlpha() - CASTING_BAR_ALPHA_STEP;
			if ( alpha > 0 ) then
				this:SetAlpha(alpha);
			else
				this.fadeOut = nil;
				this:Hide();
			end
		end
	else
		CastingBarFrame:Hide();
	end
end

function BM_ShapeshiftBar_Update()
	local numForms = GetNumShapeshiftForms();
	local fileName, name, isActive, isCastable;
	local button, icon, cooldown;
	local start, duration, enable;
	if ( numForms > 0 and BM_CONFIG[BM_PLAYERNAME_REALM].shapeshift_enable) then
		ShapeshiftBarLeft:Hide();
		ShapeshiftBarMiddle:Hide();
		ShapeshiftBarRight:Hide();
		ShapeshiftBarFrame:ClearAllPoints();
		ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "BhaldieShapeShift", "BOTTOMLEFT", 3, 8);
		ShapeshiftBarFrame:Show();
	else
		ShapeshiftBarLeft:Hide();
		ShapeshiftBarMiddle:Hide();
		ShapeshiftBarRight:Hide();
		ShapeshiftBarFrame:ClearAllPoints();
		ShapeshiftBarFrame:SetPoint("BOTTOMLEFT", "BhaldieShapeShift", "BOTTOMLEFT", 3, 8);
		ShapeshiftBarFrame:Hide();
	end
	ShapeshiftBar_UpdateState();
end

UIPARENT_MANAGED_FRAME_POSITIONS = {};
-- Frames
UIPARENT_MANAGED_FRAME_POSITIONS["MultiBarBottomLeft"] = {baseY = 17, reputation = 9, maxLevel = -5, anchorTo = "ActionButton1", point = "BOTTOMLEFT", rpoint = "TOPLEFT"};
UIPARENT_MANAGED_FRAME_POSITIONS["GroupLootFrame1"] = {baseY = 60, bottomEither = 42, pet = 42, reputation = 9};
UIPARENT_MANAGED_FRAME_POSITIONS["TutorialFrameParent"] = {baseY = 55, bottomEither = 47, pet = 42, reputation = 9};
UIPARENT_MANAGED_FRAME_POSITIONS["FramerateLabel"] = {baseY = 64, bottomEither = 42, pet = 42, reputation = 9};
UIPARENT_MANAGED_FRAME_POSITIONS["CastingBarFrame"] = {baseY = 60, bottomEither = 40, pet = 40, reputation = 9};
UIPARENT_MANAGED_FRAME_POSITIONS["ChatFrame1"] = {baseY = 85, bottomLeft = 17, pet = 17, reputation = 9, maxLevel = -5, anchorTo = "UIParent", point = "BOTTOMLEFT", rpoint = "BOTTOMLEFT", xOffset = 32};
UIPARENT_MANAGED_FRAME_POSITIONS["ChatFrame2"] = {baseY = 85, bottomRight = 17, rightLeft = -88, rightRight = -43, reputation = 9, maxLevel = -5, anchorTo = "UIParent", point = "BOTTOMRIGHT", rpoint = "BOTTOMRIGHT", xOffset = -32};
UIPARENT_MANAGED_FRAME_POSITIONS["ShapeshiftBarFrame"] = {baseY = 8, bottomLeft = 0, reputation = 0, maxLevel = 0, anchorTo = "BhaldieShapeShift", point = "BOTTOMLEFT", rpoint = "BOTTOMLEFT", xOffset = 3};

-- Vars
UIPARENT_MANAGED_FRAME_POSITIONS["CONTAINER_OFFSET_X"] = {baseX = 0, rightLeft = 90, rightRight = 45, isVar = "xAxis"};
UIPARENT_MANAGED_FRAME_POSITIONS["CONTAINER_OFFSET_Y"] = {baseY = 70, bottomRight = 27, reputation = 9, isVar = "yAxis"};
UIPARENT_MANAGED_FRAME_POSITIONS["BATTLEFIELD_TAB_OFFSET_Y"] = {baseY = 210, bottomRight = 40, reputation = 9, isVar = "yAxis"};
UIPARENT_MANAGED_FRAME_POSITIONS["PETACTIONBAR_YPOS"] = {baseY = 98, bottomLeft = 43, reputation = 9, maxLevel = -5, isVar = "yAxis"};

-- Call this function to update the positions of all frames that can appear on the right side of the screen
function BM_UIParent_ManageFramePositions()
	-- Frames that affect offsets in y axis
	local yOffsetFrames = {};
	-- Frames that affect offsets in x axis
	local xOffsetFrames = {};
	
	-- Set up flags
	if ( MultiBarBottomRight:IsShown() or MultiBarBottomLeft:IsShown() ) then
		tinsert(yOffsetFrames, "bottomEither");
	end
	if ( MultiBarBottomRight:IsShown() ) then
		tinsert(yOffsetFrames, "bottomRight");
	end
	if ( MultiBarBottomLeft:IsShown() ) then
		tinsert(yOffsetFrames, "bottomLeft");
	end
	if ( MultiBarLeft:IsShown() ) then
		tinsert(xOffsetFrames, "rightLeft");
	elseif ( MultiBarRight:IsShown() ) then
		tinsert(xOffsetFrames, "rightRight");
	end

	if ( PetActionBarFrame:IsShown() or ShapeshiftBarFrame:IsShown() ) then
		tinsert(yOffsetFrames, "pet");
	end
	if ( ReputationWatchBar:IsShown() and MainMenuExpBar:IsShown() ) then
		tinsert(yOffsetFrames, "reputation");
	end
	if ( MainMenuBarMaxLevelBar:IsShown() ) then
		tinsert(yOffsetFrames, "maxLevel");
	end
	
	-- Iterate through frames and set anchors according to the flags set
	local frame, xOffset, yOffset, anchorTo, point, rpoint;
	for index, value in UIPARENT_MANAGED_FRAME_POSITIONS do
		frame = getglobal(index);
		if ( frame ) then
			-- Always start with base as the base offset or default to zero if no "none" specified
			xOffset = 0;
			if ( value["baseX"] ) then
				xOffset = value["baseX"];
			elseif ( value["xOffset"] ) then
				xOffset = value["xOffset"];
			end
			yOffset = 0;
			if ( value["baseY"] ) then
				yOffset = value["baseY"];
			end
			
			-- Iterate through frames that affect y offsets
			local hasBottomLeft, hasPetBar;
			for flag, flagValue in yOffsetFrames do
				if ( value[flagValue] ) then
					if ( flagValue == "bottomLeft" ) then
						hasBottomLeft = 1;
					elseif ( flagValue == "pet" ) then
						hasPetBar = 1;
					end
					yOffset = yOffset + value[flagValue];
				end
			end

			if ( hasBottomLeft and hasPetBar ) then
				yOffset = yOffset + 23;
			end

			-- Iterate through frames that affect x offsets
			for flag, flagValue in xOffsetFrames do
				if ( value[flagValue] ) then
					xOffset = xOffset + value[flagValue];
				end
			end
			
			-- Set up anchoring info
			anchorTo = value["anchorTo"];
			point = value["point"];
			rpoint = value["rpoint"];
			if ( not anchorTo ) then
				anchorTo = "UIParent";
			end
			if ( not point ) then
				point = "BOTTOM";
			end
			if ( not rpoint ) then
				rpoint = "BOTTOM";
			end
			
			-- Anchor frame
			if ( value["isVar"] ) then
				if ( value["isVar"] == "xAxis" ) then
					setglobal(index, xOffset);
				else
					setglobal(index, yOffset);
				end
			else
				if ((frame == ChatFrame1 or frame == ChatFrame2) and SIMPLE_CHAT == "1") then
					frame:SetPoint(point, anchorTo, rpoint, xOffset, yOffset);
				elseif ( not(frame:IsObjectType("frame") and frame:IsUserPlaced()) ) then
					frame:SetPoint(point, anchorTo, rpoint, xOffset, yOffset);
				end
			end
		end
	end
	
	-- Custom positioning not handled by the loop
	-- Set battlefield minimap position
	if ( BattlefieldMinimapTab and not BattlefieldMinimapTab:IsUserPlaced() ) then
		BattlefieldMinimapTab:SetPoint("BOTTOMLEFT", "UIParent", "BOTTOMRIGHT", -225-CONTAINER_OFFSET_X, BATTLEFIELD_TAB_OFFSET_Y);
	end

	-- Update shapeshift bar appearance
	if ( MultiBarBottomLeft:IsShown() ) then
		ShapeshiftBarLeft:Hide();
		ShapeshiftBarRight:Hide();
		ShapeshiftBarMiddle:Hide();
		for i=1, GetNumShapeshiftForms() do
			getglobal("ShapeshiftButton"..i.."NormalTexture"):SetWidth(50);
			getglobal("ShapeshiftButton"..i.."NormalTexture"):SetHeight(50);
		end
	else
		if ( GetNumShapeshiftForms() > 2 ) then
			ShapeshiftBarMiddle:Show();
		end
		ShapeshiftBarLeft:Hide();
		ShapeshiftBarRight:Hide();
		for i=1, GetNumShapeshiftForms() do
			getglobal("ShapeshiftButton"..i.."NormalTexture"):SetWidth(64);
			getglobal("ShapeshiftButton"..i.."NormalTexture"):SetHeight(64);
		end
	end

	-- If petactionbar is already shown have to set its point is addition to changing its y target
	if ( PetActionBarFrame:IsShown() ) then
		PetActionBarFrame:SetPoint("TOPLEFT", MainMenuBar, "BOTTOMLEFT", PETACTIONBAR_XPOS, PETACTIONBAR_YPOS);
	end

	-- Setup y anchors
	local anchorY = 0;
	QuestTimerFrame:SetPoint("TOPRIGHT", "MinimapCluster", "BOTTOMRIGHT", -CONTAINER_OFFSET_X, anchorY);
	if ( QuestTimerFrame:IsShown() ) then
		anchorY = anchorY - QuestTimerFrame:GetHeight();
	end
	-- Setup durability offset
	if ( DurabilityFrame ) then
		local durabilityOffset = 0;
		if ( DurabilityShield:IsShown() or DurabilityOffWeapon:IsShown() or DurabilityRanged:IsShown() ) then
			durabilityOffset = 20;
		end
		DurabilityFrame:SetPoint("TOPRIGHT", "MinimapCluster", "BOTTOMRIGHT", -CONTAINER_OFFSET_X-durabilityOffset, anchorY);
		if ( DurabilityFrame:IsShown() ) then
			anchorY = anchorY - DurabilityFrame:GetHeight();
		end
	end
	
	QuestWatchFrame:SetPoint("TOPRIGHT", "MinimapCluster", "BOTTOMRIGHT", -CONTAINER_OFFSET_X, anchorY);

	-- Update chat dock since the dock could have moved
	FCF_DockUpdate();
end